@@ -1,12 +1,14 @@
 package Color::ANSI::Util;
 
-our $DATE = '2018-12-02'; # DATE
-our $VERSION = '0.162'; # VERSION
+our $DATE = '2019-08-20'; # DATE
+our $VERSION = '0.163'; # VERSION
 
 use 5.010001;
 use strict;
 use warnings;
 
+use Color::RGB::Util qw(rgb_diff);
+
 require Exporter;
 our @ISA       = qw(Exporter);
 our @EXPORT_OK = qw(
@@ -58,9 +60,8 @@ my %ansi16 = (
     15 => 'ffffff',
 );
 my @revansi16;
-for (sort {$a<=>$b} keys %ansi16) {
-    $ansi16{$_} =~ /(..)(..)(..)/;
-    push @revansi16, [hex($1), hex($2), hex($3), $_];
+for my $idx (sort {$a<=>$b} keys %ansi16) {
+    push @revansi16, [$ansi16{$idx}, $idx];
 }
 
 my %ansi256 = (
@@ -109,9 +110,8 @@ my %ansi256 = (
     250 => 'bcbcbc', 251 => 'c6c6c6', 252 => 'd0d0d0', 253 => 'dadada', 254 => 'e4e4e4', 255 => 'eeeeee',
 );
 my @revansi256;
-for (sort {$a<=>$b} keys %ansi256) {
-    $ansi256{$_} =~ /(..)(..)(..)/;
-    push @revansi256, [hex($1), hex($2), hex($3), $_];
+for my $idx (sort {$a<=>$b} keys %ansi256) {
+    push @revansi256, [$ansi256{$idx}, $idx];
 }
 
 $SPEC{ansi16_to_rgb} = {
@@ -174,22 +174,14 @@ sub ansi16_to_rgb {
 sub _rgb_to_indexed {
     my ($rgb, $table) = @_;
 
-    $rgb =~ /^#?([0-9A-Fa-f]{2})([0-9A-Fa-f]{2})([0-9A-Fa-f]{2})$/
-        or die "Invalid RGB input '$rgb'";
-    my $r = hex($1);
-    my $g = hex($2);
-    my $b = hex($3);
-
-    my ($minsqdist, $res);
+    my ($smallest_diff, $res);
     for my $e (@$table) {
-        my $sqdist =
-            abs($e->[0]-$r)**2 + abs($e->[1]-$g)**2 + abs($e->[2]-$b)**2;
+        my $diff = rgb_diff($rgb, $e->[0], 'hsv_hue1');
         # exact match, return immediately
-        return $e->[3] if $sqdist == 0;
-        if (!defined($minsqdist) || $minsqdist > $sqdist) {
-            #say "D:sqdist=$sqdist";
-            $minsqdist = $sqdist;
-            $res = $e->[3];
+        return $e->[1] if $diff == 0;
+        if (!defined($smallest_diff) || $smallest_diff > $diff) {
+            $smallest_diff = $diff;
+            $res = $e->[1];
         }
     }
     return $res;
@@ -550,7 +542,7 @@ Color::ANSI::Util - Routines for dealing with ANSI colors
 
 =head1 VERSION
 
-This document describes version 0.162 of Color::ANSI::Util (from Perl distribution Color-ANSI-Util), released on 2018-12-02.
+This document describes version 0.163 of Color::ANSI::Util (from Perl distribution Color-ANSI-Util), released on 2019-08-20.
 
 =head1 SYNOPSIS
 
@@ -613,6 +605,7 @@ Arguments ('*' denotes required arguments):
 Return value:  (color::rgb24)
 
 
+
 =head2 ansi256_to_rgb
 
 Usage:
@@ -634,6 +627,7 @@ Arguments ('*' denotes required arguments):
 Return value:  (color::rgb24)
 
 
+
 =head2 rgb_to_ansi16
 
 Usage:
@@ -655,6 +649,7 @@ Arguments ('*' denotes required arguments):
 Return value:  (color::ansi16)
 
 
+
 =head2 rgb_to_ansi16_bg_code
 
 Usage:
@@ -676,6 +671,7 @@ Arguments ('*' denotes required arguments):
 Return value:  (str)
 
 
+
 =head2 rgb_to_ansi16_fg_code
 
 Usage:
@@ -697,6 +693,7 @@ Arguments ('*' denotes required arguments):
 Return value:  (str)
 
 
+
 =head2 rgb_to_ansi24b_bg_code
 
 Usage:
@@ -718,6 +715,7 @@ Arguments ('*' denotes required arguments):
 Return value:  (str)
 
 
+
 =head2 rgb_to_ansi24b_fg_code
 
 Usage:
@@ -739,6 +737,7 @@ Arguments ('*' denotes required arguments):
 Return value:  (str)
 
 
+
 =head2 rgb_to_ansi256
 
 Usage:
@@ -760,6 +759,7 @@ Arguments ('*' denotes required arguments):
 Return value:  (color::ansi256)
 
 
+
 =head2 rgb_to_ansi256_bg_code
 
 Usage:
@@ -781,6 +781,7 @@ Arguments ('*' denotes required arguments):
 Return value:  (str)
 
 
+
 =head2 rgb_to_ansi256_fg_code
 
 Usage:
@@ -802,6 +803,7 @@ Arguments ('*' denotes required arguments):
 Return value:  (str)
 
 
+
 =head2 rgb_to_ansi_bg_code
 
 Usage:
@@ -830,6 +832,7 @@ Arguments ('*' denotes required arguments):
 Return value:  (str)
 
 
+
 =head2 rgb_to_ansi_fg_code
 
 Usage:
@@ -857,6 +860,7 @@ Arguments ('*' denotes required arguments):
 
 Return value:  (str)
 
+
 =head2 ansi16fg($rgb) => STR
 
 Alias for rgb_to_ansi16_fg_code().
@@ -946,7 +950,7 @@ perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2018, 2017, 2016, 2015, 2014, 2013 by perlancar@cpan.org.
+This software is copyright (c) 2019, 2018, 2017, 2016, 2015, 2014, 2013 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
